@@ -1090,6 +1090,10 @@ tracker_data_blank_buffer_flush (GError **error)
 	/* generate hash uri from data to find resource
 	   assumes no collisions due to generally little contents of anonymous nodes */
 	for (i = 0; i < blank_buffer.predicates->len; i++) {
+		if (blank_buffer.graphs->data[i] != NULL) {
+			g_checksum_update (checksum, g_array_index (blank_buffer.graphs, guchar *, i), -1);
+		}
+
 		g_checksum_update (checksum, g_array_index (blank_buffer.predicates, guchar *, i), -1);
 		g_checksum_update (checksum, g_array_index (blank_buffer.objects, guchar *, i), -1);
 	}
@@ -2508,6 +2512,16 @@ tracker_data_commit_transaction (GError **error)
 
 	tracker_data_update_buffer_flush (&actual_error);
 	if (actual_error) {
+		tracker_data_rollback_transaction ();
+		g_propagate_error (error, actual_error);
+		return;
+	}
+
+	tracker_db_interface_end_db_transaction (iface,
+	                                         &actual_error);
+
+	if (actual_error) {
+		tracker_data_rollback_transaction ();
 		g_propagate_error (error, actual_error);
 		return;
 	}
@@ -2545,8 +2559,6 @@ tracker_data_commit_transaction (GError **error)
 	}
 #endif
 
-	tracker_db_interface_end_db_transaction (iface);
-
 	tracker_db_interface_execute_query (iface, NULL, "PRAGMA cache_size = %d", TRACKER_DB_CACHE_SIZE_DEFAULT);
 
 	g_hash_table_remove_all (update_buffer.resources);
@@ -2573,6 +2585,7 @@ void
 tracker_data_rollback_transaction (void)
 {
 	TrackerDBInterface *iface;
+	GError *ignorable = NULL;
 
 	g_return_if_fail (in_transaction);
 
@@ -2582,28 +2595,37 @@ tracker_data_rollback_transaction (void)
 	iface = tracker_db_manager_get_db_interface ();
 
 	tracker_data_update_buffer_clear ();
-	tracker_db_interface_execute_query (iface, NULL, "ROLLBACK");
-	tracker_db_interface_execute_query (iface, NULL, "PRAGMA cache_size = %d", TRACKER_DB_CACHE_SIZE_DEFAULT);
-	tracker_db_journal_rollback_transaction ();
 
-	if (rollback_callbacks) {
-		guint n;
-		for (n = 0; n < rollback_callbacks->len; n++) {
-			TrackerCommitDelegate *delegate;
-			delegate = g_ptr_array_index (rollback_callbacks, n);
-			delegate->callback (TRUE, delegate->user_data);
+	tracker_db_interface_execute_query (iface, &ignorable, "ROLLBACK");
+
+	if (ignorable) {
+		g_error_free (ignorable);
+	}
+
+	tracker_db_interface_execute_query (iface, NULL, "PRAGMA cache_size = %d", TRACKER_DB_CACHE_SIZE_DEFAULT);
+
+	if (!in_journal_replay) {
+		tracker_db_journal_rollback_transaction ();
+
+		if (rollback_callbacks) {
+			guint n;
+			for (n = 0; n < rollback_callbacks->len; n++) {
+				TrackerCommitDelegate *delegate;
+				delegate = g_ptr_array_index (rollback_callbacks, n);
+				delegate->callback (TRUE, delegate->user_data);
+			}
 		}
 	}
 }
 
-static GPtrArray *
+static GVariant *
 update_sparql (const gchar  *update,
                gboolean      blank,
                GError      **error)
 {
 	GError *actual_error = NULL;
 	TrackerSparqlQuery *sparql_query;
-	GPtrArray *blank_nodes;
+	GVariant *blank_nodes;
 
 	g_return_val_if_fail (update != NULL, NULL);
 
@@ -2625,7 +2647,6 @@ update_sparql (const gchar  *update,
 
 	tracker_data_commit_transaction (&actual_error);
 	if (actual_error) {
-		tracker_data_rollback_transaction ();
 		g_propagate_error (error, actual_error);
 		return NULL;
 	}
@@ -2640,7 +2661,7 @@ tracker_data_update_sparql (const gchar  *update,
 	update_sparql (update, FALSE, error);
 }
 
-GPtrArray *
+GVariant *
 tracker_data_update_sparql_blank (const gchar  *update,
                                   GError      **error)
 {
@@ -2667,9 +2688,10 @@ tracker_data_sync (void)
 }
 
 void
-tracker_data_replay_journal (TrackerBusyCallback  busy_callback,
-                             gpointer             busy_user_data,
-                             const gchar         *busy_status)
+tracker_data_replay_journal (TrackerBusyCallback   busy_callback,
+                             gpointer              busy_user_data,
+                             const gchar          *busy_status,
+                             GError              **error)
 {
 	GError *journal_error = NULL;
 	TrackerProperty *rdf_type = NULL;
@@ -2719,8 +2741,14 @@ tracker_data_replay_journal (TrackerBusyCallback  busy_callback,
 
 			tracker_data_commit_transaction (&new_error);
 			if (new_error) {
-				g_warning ("Journal replay error: '%s'", new_error->message);
-				g_clear_error (&new_error);
+				/* Out of disk is an unrecoverable fatal error */
+				if (g_error_matches (new_error, TRACKER_DB_INTERFACE_ERROR, TRACKER_DB_NO_SPACE)) {
+					g_propagate_error (error, new_error);
+					return;
+				} else {
+					g_warning ("Journal replay error: '%s'", new_error->message);
+					g_clear_error (&new_error);
+				}
 			}
 		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT) {
 			GError *new_error = NULL;
