@@ -22,6 +22,7 @@
 #define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
 #define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
 #define g_marshal_value_peek_object(v)   g_value_get_object (v)
+#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
 #else /* !G_ENABLE_DEBUG */
 /* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
  *          Do not access GValues directly in your code. Instead, use the
@@ -45,6 +46,7 @@
 #define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
 #define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
 #define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
+#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
 #endif /* !G_ENABLE_DEBUG */
 
 
@@ -368,7 +370,50 @@ tracker_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* BOOL:OBJECT,OBJECT,OBJECT (tracker-marshal.list:9) */
+/* VOID:STRING,STRING,STRING,BOOLEAN,BOOLEAN (tracker-marshal.list:9) */
+void
+tracker_marshal_VOID__STRING_STRING_STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
+                                                            GValue       *return_value G_GNUC_UNUSED,
+                                                            guint         n_param_values,
+                                                            const GValue *param_values,
+                                                            gpointer      invocation_hint G_GNUC_UNUSED,
+                                                            gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__STRING_STRING_STRING_BOOLEAN_BOOLEAN) (gpointer     data1,
+                                                                           gpointer     arg_1,
+                                                                           gpointer     arg_2,
+                                                                           gpointer     arg_3,
+                                                                           gboolean     arg_4,
+                                                                           gboolean     arg_5,
+                                                                           gpointer     data2);
+  register GMarshalFunc_VOID__STRING_STRING_STRING_BOOLEAN_BOOLEAN callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 6);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__STRING_STRING_STRING_BOOLEAN_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_string (param_values + 1),
+            g_marshal_value_peek_string (param_values + 2),
+            g_marshal_value_peek_string (param_values + 3),
+            g_marshal_value_peek_boolean (param_values + 4),
+            g_marshal_value_peek_boolean (param_values + 5),
+            data2);
+}
+
+/* BOOL:OBJECT,OBJECT,OBJECT (tracker-marshal.list:10) */
 void
 tracker_marshal_BOOLEAN__OBJECT_OBJECT_OBJECT (GClosure     *closure,
                                                GValue       *return_value G_GNUC_UNUSED,
@@ -411,7 +456,7 @@ tracker_marshal_BOOLEAN__OBJECT_OBJECT_OBJECT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOL:OBJECT,OBJECT (tracker-marshal.list:10) */
+/* BOOL:OBJECT,OBJECT (tracker-marshal.list:11) */
 void
 tracker_marshal_BOOLEAN__OBJECT_OBJECT (GClosure     *closure,
                                         GValue       *return_value G_GNUC_UNUSED,
@@ -452,7 +497,7 @@ tracker_marshal_BOOLEAN__OBJECT_OBJECT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOL:OBJECT,POINTER (tracker-marshal.list:11) */
+/* BOOL:OBJECT,POINTER (tracker-marshal.list:12) */
 void
 tracker_marshal_BOOLEAN__OBJECT_POINTER (GClosure     *closure,
                                          GValue       *return_value G_GNUC_UNUSED,
@@ -493,7 +538,7 @@ tracker_marshal_BOOLEAN__OBJECT_POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOL:OBJECT (tracker-marshal.list:12) */
+/* BOOL:OBJECT (tracker-marshal.list:13) */
 void
 tracker_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                  GValue       *return_value G_GNUC_UNUSED,
