@@ -2,18 +2,18 @@
  * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
  * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
@@ -67,8 +67,9 @@ ext_block_append(ExtBlock *extBlock,
 		 unsigned char extData[])
 {
 	extBlock->bytes = realloc(extBlock->bytes,extBlock->byteCount+len);
-	if (extBlock->bytes == NULL)
+	if (extBlock->bytes == NULL) {
 		return (GIF_ERROR);
+	}
 
 	memcpy(&(extBlock->bytes[extBlock->byteCount]), &extData[0], len);
 	extBlock->byteCount += len;
@@ -82,25 +83,29 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 	       GifFileType          *gifFile,
                const gchar          *uri)
 {
-	GifRecordType  RecordType;
-	int            frameheight;
-	int            framewidth;
+	GifRecordType RecordType;
+	int frameheight;
+	int framewidth;
 	unsigned char *framedata = NULL;
-	GPtrArray     *keywords;
-	guint          i;
-	int            status;
+	GPtrArray *keywords;
+	guint i;
+	int status;
 	MergeData md = { 0 };
 	GifData   gd = { 0 };
 	TrackerXmpData *xd = NULL;
 
 	do {
+		GifByteType *ExtData;
+		int ExtCode;
+		ExtBlock extBlock;
+
 		if (DGifGetRecordType(gifFile, &RecordType) == GIF_ERROR) {
 			PrintGifError();
 			return;
 		}
 
 		switch (RecordType) {
-		case IMAGE_DESC_RECORD_TYPE:
+			case IMAGE_DESC_RECORD_TYPE:
 			if (DGifGetImageDesc(gifFile) == GIF_ERROR) {
 				PrintGifError();
 				return;
@@ -122,76 +127,70 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 
 			g_free (framedata);
 
-			break;
+		break;
 		case EXTENSION_RECORD_TYPE:
-			{
-				GifByteType   *ExtData;
-				int            ExtCode;
-				ExtBlock       extBlock;
+			extBlock.bytes = NULL;
+			extBlock.byteCount = 0;
 
-				extBlock.bytes = NULL;
-				extBlock.byteCount = 0;
-
-				if ((status = DGifGetExtension (gifFile, &ExtCode, &ExtData)) != GIF_OK) {
-					g_warning ("Problem getting the extension");
-					return;
-				}
+			if ((status = DGifGetExtension (gifFile, &ExtCode, &ExtData)) != GIF_OK) {
+				g_warning ("Problem getting the extension");
+				return;
+			}
 #if defined(HAVE_EXEMPI)
-				if (ExtData && *ExtData &&
-				    strncmp (&ExtData[1],"XMP Data",8) == 0) {
-					while (ExtData != NULL && status == GIF_OK ) {
-						if ((status = DGifGetExtensionNext (gifFile, &ExtData)) == GIF_OK) {
-							if (ExtData != NULL) {
-								if (ext_block_append (&extBlock, ExtData[0]+1, (char *) &(ExtData[0])) != GIF_OK) {
-									g_warning ("Problem with extension data");
-									return;
-								}
+			if (ExtData && *ExtData &&
+			    strncmp (&ExtData[1],"XMP Data",8) == 0) {
+				while (ExtData != NULL && status == GIF_OK ) {
+					if ((status = DGifGetExtensionNext (gifFile, &ExtData)) == GIF_OK) {
+						if (ExtData != NULL) {
+							if (ext_block_append (&extBlock, ExtData[0]+1, (char *) &(ExtData[0])) != GIF_OK) {
+								g_warning ("Problem with extension data");
+								return;
 							}
 						}
 					}
-
-					xd = tracker_xmp_new (extBlock.bytes,
-							      extBlock.byteCount-XMP_MAGIC_TRAILER_LENGTH,
-							      uri);
-
-					g_free (extBlock.bytes);
-				} else
-#endif
-				/* See Section 24. Comment Extension. in the GIF format definition */
-				if (ExtCode == EXTENSION_RECORD_COMMENT_BLOCK_CODE &&
-				    ExtData && *ExtData) {
-					guint block_count = 0;
-
-					/* Merge all blocks */
-					do {
-						block_count++;
-
-						g_debug ("Comment Extension block found (#%u, %u bytes)",
-						         block_count,
-						         ExtData[0]);
-						if (ext_block_append (&extBlock, ExtData[0], (char *) &(ExtData[1])) != GIF_OK) {
-							g_warning ("Problem with Comment extension data");
-							return;
-						}
-					} while (((status = DGifGetExtensionNext(gifFile, &ExtData)) == GIF_OK) &&
-					         ExtData != NULL);
-
-					/* Add last NUL byte */
-					g_debug ("Comment Extension blocks found (%u) with %u bytes",
-					         block_count,
-					         extBlock.byteCount);
-					extBlock.bytes = g_realloc (extBlock.bytes, extBlock.byteCount + 1);
-					extBlock.bytes[extBlock.byteCount] = '\0';
-
-					/* Set commentt */
-					gd.comment = extBlock.bytes;
-				} else {
-					do {
-						status = DGifGetExtensionNext(gifFile, &ExtData);
-					} while ( status == GIF_OK && ExtData != NULL);
 				}
+
+				xd = tracker_xmp_new (extBlock.bytes,
+						      extBlock.byteCount-XMP_MAGIC_TRAILER_LENGTH,
+						      uri);
+
+				g_free (extBlock.bytes);
+			} else
+#endif
+			/* See Section 24. Comment Extension. in the GIF format definition */
+			if (ExtCode == EXTENSION_RECORD_COMMENT_BLOCK_CODE &&
+			    ExtData && *ExtData) {
+				guint block_count = 0;
+
+				/* Merge all blocks */
+				do {
+					block_count++;
+
+					g_debug ("Comment Extension block found (#%u, %u bytes)",
+					         block_count,
+					         ExtData[0]);
+					if (ext_block_append (&extBlock, ExtData[0], (char *) &(ExtData[1])) != GIF_OK) {
+						g_warning ("Problem with Comment extension data");
+						return;
+					}
+				} while (((status = DGifGetExtensionNext(gifFile, &ExtData)) == GIF_OK) &&
+				         ExtData != NULL);
+
+				/* Add last NUL byte */
+				g_debug ("Comment Extension blocks found (%u) with %u bytes",
+				         block_count,
+				         extBlock.byteCount);
+				extBlock.bytes = g_realloc (extBlock.bytes, extBlock.byteCount + 1);
+				extBlock.bytes[extBlock.byteCount] = '\0';
+
+				/* Set commentt */
+				gd.comment = extBlock.bytes;
+			} else {
+				do {
+					status = DGifGetExtensionNext(gifFile, &ExtData);
+				} while ( status == GIF_OK && ExtData != NULL);
 			}
-			break;
+		break;
 		case TERMINATE_RECORD_TYPE:
 			break;
 		default:
@@ -229,10 +228,10 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		g_free (uri);
 	}
 
-	if (md.date) {
-		tracker_sparql_builder_predicate (metadata, "nie:contentCreated");
-		tracker_sparql_builder_object_unvalidated (metadata, md.date);
-	}
+	tracker_guarantee_date_from_file_mtime (metadata,
+	                                        "nie:contentCreated",
+	                                        md.date,
+	                                        uri);
 
 	if (xd->description) {
 		tracker_sparql_builder_predicate (metadata, "nie:description");
@@ -270,10 +269,10 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 		g_free (equip_uri);
 	}
 
-	if (md.title) {
-		tracker_sparql_builder_predicate (metadata, "nie:title");
-		tracker_sparql_builder_object_unvalidated (metadata, md.title);
-	}
+	tracker_guarantee_title_from_file (metadata,
+	                                   "nie:title",
+	                                   md.title,
+	                                   uri);
 
 	if (md.artist) {
 		gchar *uri = tracker_sparql_escape_uri_printf ("urn:contact:%s", md.artist);
@@ -421,38 +420,18 @@ read_metadata (TrackerSparqlBuilder *preupdate,
 	if (xd->address || xd->country || xd->city) {
 		gchar *addruri;
 
-		tracker_sparql_builder_predicate (metadata, "mlo:location");
+		tracker_sparql_builder_predicate (metadata, "slo:location");
 
-		tracker_sparql_builder_object_blank_open (metadata); /* GeoPoint */
+		tracker_sparql_builder_object_blank_open (metadata); /* GeoLocation */
 		tracker_sparql_builder_predicate (metadata, "a");
-		tracker_sparql_builder_object (metadata, "mlo:GeoPoint");
-
-		if (xd->address) {
-			tracker_sparql_builder_predicate (metadata, "mlo:address");
-			tracker_sparql_builder_object_unvalidated (metadata, xd->address);
-		}
-
-		if (xd->state) {
-			tracker_sparql_builder_predicate (metadata, "mlo:state");
-			tracker_sparql_builder_object_unvalidated (metadata, xd->state);
-		}
-
-		if (xd->city) {
-			tracker_sparql_builder_predicate (metadata, "mlo:city");
-			tracker_sparql_builder_object_unvalidated (metadata, xd->city);
-		}
-
-		if (xd->country) {
-			tracker_sparql_builder_predicate (metadata, "mlo:country");
-			tracker_sparql_builder_object_unvalidated (metadata, xd->country);
-		}
+		tracker_sparql_builder_object (metadata, "slo:GeoLocation");
 
 		addruri = tracker_sparql_get_uuid_urn ();
 
-		tracker_sparql_builder_predicate (metadata, "mlo:asPostalAddress");
+		tracker_sparql_builder_predicate (metadata, "slo:postalAddress");
 		tracker_sparql_builder_object_iri (metadata, addruri);
 
-		tracker_sparql_builder_object_blank_close (metadata); /* GeoPoint */
+		tracker_sparql_builder_object_blank_close (metadata); /* GeoLocation */
 
 		tracker_sparql_builder_insert_open (preupdate, NULL);
 		tracker_sparql_builder_subject_iri (preupdate, addruri);
@@ -513,9 +492,9 @@ extract_gif (const gchar          *uri,
              TrackerSparqlBuilder *preupdate,
              TrackerSparqlBuilder *metadata)
 {
-	goffset      size;
+	goffset size;
 	GifFileType *gifFile = NULL;
-	gchar       *filename;
+	gchar *filename;
 
 	filename = g_filename_from_uri (uri, NULL, NULL);
 	size = tracker_file_get_size (filename);
