@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
- * Copyright (C) 2008-2009, Nokia <ivan.frade@nokia.com>
+ * Copyright (C) 2008-2011, Nokia <ivan.frade@nokia.com>
  * Copyright (C) 2010, Amit Aggarwal <amitcs06@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
 #include <string.h>
 
 #include <glib.h>
-#include <poppler.h>
+#include <glib/poppler.h>
 
 #include <libtracker-common/tracker-date-time.h>
 #include <libtracker-common/tracker-utils.h>
@@ -75,7 +75,7 @@ read_toc (PopplerIndexIter  *index,
 
 		switch (action->type) {
 			case POPPLER_ACTION_GOTO_DEST: {
-				PopplerActionGotoDest *ag = (PopplerActionGotoDest*) action;
+				PopplerActionGotoDest *ag = (PopplerActionGotoDest *)action;
 				PopplerDest *agd = ag->dest;
 
 				if (!tracker_is_empty_string (ag->title)) {
@@ -90,7 +90,7 @@ read_toc (PopplerIndexIter  *index,
 			}
 
 			case POPPLER_ACTION_LAUNCH: {
-				PopplerActionLaunch *al = (PopplerActionLaunch*) action;
+				PopplerActionLaunch *al = (PopplerActionLaunch *)action;
 
 				if (!tracker_is_empty_string (al->title)) {
 					g_string_append_printf (*toc, "%s ", al->title);
@@ -108,7 +108,7 @@ read_toc (PopplerIndexIter  *index,
 			}
 
 			case POPPLER_ACTION_URI: {
-				PopplerActionUri *au = (PopplerActionUri*) action;
+				PopplerActionUri *au = (PopplerActionUri *)action;
 
 				if (!tracker_is_empty_string (au->uri)) {
 					g_string_append_printf (*toc, "%s ", au->uri);
@@ -118,7 +118,7 @@ read_toc (PopplerIndexIter  *index,
 			}
 
 			case POPPLER_ACTION_NAMED: {
-				PopplerActionNamed *an = (PopplerActionNamed*) action;
+				PopplerActionNamed *an = (PopplerActionNamed *)action;
 
 				if (!tracker_is_empty_string (an->title)) {
 					g_string_append_printf (*toc, "%s, ", an->title);
@@ -132,7 +132,7 @@ read_toc (PopplerIndexIter  *index,
 			}
 
 			case POPPLER_ACTION_MOVIE: {
-				PopplerActionNamed *am = (PopplerActionNamed*) action;
+				PopplerActionMovie *am = (PopplerActionMovie *)action;
 
 				if (!tracker_is_empty_string (am->title)) {
 					g_string_append_printf (*toc, "%s ", am->title);
@@ -144,6 +144,8 @@ read_toc (PopplerIndexIter  *index,
 			case POPPLER_ACTION_NONE:
 			case POPPLER_ACTION_UNKNOWN:
 			case POPPLER_ACTION_GOTO_REMOTE:
+			case POPPLER_ACTION_RENDITION:
+			case POPPLER_ACTION_OCG_STATE:
 				/* Do nothing */
 				break;
 		}
@@ -180,83 +182,66 @@ read_outline (PopplerDocument      *document,
 	}
 }
 
-static void
-insert_keywords (TrackerSparqlBuilder *metadata,
-                 gchar                *keywords)
-{
-	char *saveptr, *p;
-	size_t len;
-
-	p = keywords;
-	keywords = strchr (keywords, '"');
-
-	if (keywords) {
-		keywords++;
-	} else {
-		keywords = p;
-	}
-
-	len = strlen (keywords);
-	if (keywords[len - 1] == '"') {
-		keywords[len - 1] = '\0';
-	}
-
-	for (p = strtok_r (keywords, ",; ", &saveptr);
-	     p;
-	     p = strtok_r (NULL, ",; ", &saveptr)) {
-		tracker_sparql_builder_predicate (metadata, "nao:hasTag");
-
-		tracker_sparql_builder_object_blank_open (metadata);
-		tracker_sparql_builder_predicate (metadata, "a");
-		tracker_sparql_builder_object (metadata, "nao:Tag");
-
-		tracker_sparql_builder_predicate (metadata, "nao:prefLabel");
-		tracker_sparql_builder_object_unvalidated (metadata, p);
-
-		tracker_sparql_builder_object_blank_close (metadata);
-	}
-}
-
 static gchar *
 extract_content (PopplerDocument *document,
-                 guint            n_words)
+                 gsize            n_bytes)
 {
-	PopplerPage *page;
-	PopplerRectangle rect;
+	gint n_pages, i = 0;
 	GString *string;
-	gint n_pages, i, words;
-	gchar *text, *t;
+	GTimer *timer;
+	gsize remaining_bytes = n_bytes;
 
 	n_pages = poppler_document_get_n_pages (document);
 	string = g_string_new ("");
-	words = i = 0;
+	timer = g_timer_new ();
 
-	while (i < n_pages && words < n_words) {
-		gint normalized_words;
+	while (i < n_pages &&
+	       remaining_bytes > 0 &&
+	       g_timer_elapsed (timer, NULL) < 5) {
+		PopplerPage *page;
+		gsize written_bytes;
+		gchar *text;
 
 		page = poppler_document_get_page (document, i);
 		i++;
 
-		rect.x1 = rect.y1 = 0;
-		poppler_page_get_size (page, &rect.x2, &rect.y2);
+		text = poppler_page_get_text (page);
 
-		text = poppler_page_get_text (page, POPPLER_SELECTION_WORD, &rect);
-		t = tracker_text_normalize (text, n_words - words, &normalized_words);
+		if (!text) {
+			g_object_unref (page);
+			continue;
+		}
 
-		words += normalized_words;
-		g_string_append (string, t);
+		if (tracker_text_validate_utf8 (text,
+		                                MIN (strlen (text), remaining_bytes),
+		                                &string,
+		                                &written_bytes)) {
+			g_string_append_c (string, ' ');
+		}
+
+		remaining_bytes -= written_bytes;
+
+		g_debug ("Extracted %" G_GSIZE_FORMAT " bytes from page %d, "
+		         "%" G_GSIZE_FORMAT " bytes remaining",
+		         written_bytes, i, remaining_bytes);
 
 		g_free (text);
-		g_free (t);
 		g_object_unref (page);
 	}
 
+	g_debug ("Content extraction finished: %d/%d pages indexed in %lf seconds, "
+	         "%" G_GSIZE_FORMAT " bytes extracted",
+	         i, n_pages, g_timer_elapsed (timer, NULL), (n_bytes - remaining_bytes));
+
+	g_timer_destroy (timer);
+
 	return g_string_free (string, FALSE);
 }
 
 static void
 write_pdf_data (PDFData               data,
-                TrackerSparqlBuilder *metadata)
+                TrackerSparqlBuilder *metadata,
+                GPtrArray            *keywords)
 {
 	if (!tracker_is_empty_string (data.title)) {
 		tracker_sparql_builder_predicate (metadata, "nie:title");
@@ -284,7 +269,7 @@ write_pdf_data (PDFData               data,
 	}
 
 	if (!tracker_is_empty_string (data.keywords)) {
-		insert_keywords (metadata, data.keywords);
+		tracker_keywords_parse (keywords, data.keywords);
 	}
 }
 
@@ -293,7 +278,7 @@ extract_pdf (const gchar          *uri,
              TrackerSparqlBuilder *preupdate,
              TrackerSparqlBuilder *metadata)
 {
-	TrackerFTSConfig *fts_config;
+	TrackerConfig *config;
 	GTime creation_date;
 	GError *error = NULL;
 	TrackerXmpData *xd = NULL;
@@ -302,7 +287,9 @@ extract_pdf (const gchar          *uri,
 	PopplerDocument *document;
 	gchar *xml = NULL;
 	gchar *content;
-	guint n_words;
+	guint n_bytes;
+	GPtrArray *keywords;
+	guint i;
 
 	g_type_init ();
 
@@ -342,41 +329,31 @@ extract_pdf (const gchar          *uri,
 	              "subject", &pd.subject,
 	              "keywords", &pd.keywords,
 	              "creation-date", &creation_date,
+	              "metadata", &xml,
 	              NULL);
 
-	/* metadata property not present in older poppler versions */
-	if (g_object_class_find_property (G_OBJECT_GET_CLASS (document), "metadata")) {
-		g_object_get (document, "metadata", &xml, NULL);
-	}
-
 	if (creation_date > 0) {
 		pd.creation_date = tracker_date_to_string ((time_t) creation_date);
 	}
 
-	if (xml) {
-		xd = tracker_xmp_new (xml, strlen (xml), uri);
-
-		if (!xd) {
-			xd = g_new0 (TrackerXmpData, 1);
-		}
-
-		g_free (xml);
-		xml = NULL;
+	keywords = g_ptr_array_new ();
 
+	if (xml &&
+	    (xd = tracker_xmp_new (xml, strlen (xml), uri)) != NULL) {
 		/* The casts here are well understood and known */
-		md.title = (gchar *) tracker_coalesce_strip (3, pd.title, xd->title, xd->title2, xd->pdf_title);
+		md.title = (gchar *) tracker_coalesce_strip (4, pd.title, xd->title, xd->title2, xd->pdf_title);
 		md.subject = (gchar *) tracker_coalesce_strip (2, pd.subject, xd->subject);
 		md.date = (gchar *) tracker_coalesce_strip (3, pd.creation_date, xd->date, xd->time_original);
 		md.author = (gchar *) tracker_coalesce_strip (2, pd.author, xd->creator);
 
-		write_pdf_data (md, metadata);
+		write_pdf_data (md, metadata, keywords);
 
 		if (xd->keywords) {
-			insert_keywords (metadata, xd->keywords);
+			tracker_keywords_parse (keywords, xd->keywords);
 		}
 
 		if (xd->pdf_keywords) {
-			insert_keywords (metadata, xd->pdf_keywords);
+			tracker_keywords_parse (keywords, xd->pdf_keywords);
 		}
 
 		if (xd->publisher) {
@@ -430,18 +407,30 @@ extract_pdf (const gchar          *uri,
 		}
 
 		if (xd->make || xd->model) {
-			gchar *camera;
+			gchar *equip_uri;
 
-			if ((xd->make == NULL || xd->model == NULL) ||
-			    (xd->make && xd->model && strstr (xd->model, xd->make) == NULL)) {
-				camera = tracker_merge_const (" ", 2, xd->make, xd->model);
-			} else {
-				camera = g_strdup (xd->model);
+			equip_uri = tracker_sparql_escape_uri_printf ("urn:equipment:%s:%s:",
+			                                              xd->make ? xd->make : "",
+			                                              xd->model ? xd->model : "");
+
+			tracker_sparql_builder_insert_open (preupdate, NULL);
+			tracker_sparql_builder_subject_iri (preupdate, equip_uri);
+			tracker_sparql_builder_predicate (preupdate, "a");
+			tracker_sparql_builder_object (preupdate, "nfo:Equipment");
+
+			if (xd->make) {
+				tracker_sparql_builder_predicate (preupdate, "nfo:manufacturer");
+				tracker_sparql_builder_object_unvalidated (preupdate, xd->make);
 			}
 
-			tracker_sparql_builder_predicate (metadata, "nmm:camera");
-			tracker_sparql_builder_object_unvalidated (metadata, camera);
-			g_free (camera);
+			if (xd->model) {
+				tracker_sparql_builder_predicate (preupdate, "nfo:model");
+				tracker_sparql_builder_object_unvalidated (preupdate, xd->model);
+			}
+			tracker_sparql_builder_insert_close (preupdate);
+			tracker_sparql_builder_predicate (metadata, "nfo:equipment");
+			tracker_sparql_builder_object_iri (metadata, equip_uri);
+			g_free (equip_uri);
 		}
 
 		if (xd->orientation) {
@@ -522,58 +511,83 @@ extract_pdf (const gchar          *uri,
 		}
 
 		if (xd->address || xd->country || xd->city) {
-			tracker_sparql_builder_predicate (metadata, "mlo:location");
+			gchar *addruri;
+
+			tracker_sparql_builder_predicate (metadata, "slo:location");
 
 			tracker_sparql_builder_object_blank_open (metadata);
 			tracker_sparql_builder_predicate (metadata, "a");
-			tracker_sparql_builder_object (metadata, "mlo:GeoPoint");
+			tracker_sparql_builder_object (metadata, "slo:GeoLocation"); /* GeoLocation */
+
+			addruri = tracker_sparql_get_uuid_urn ();
+
+			tracker_sparql_builder_predicate (metadata, "slo:postalAddress");
+			tracker_sparql_builder_object_iri (metadata, addruri);
+
+			tracker_sparql_builder_object_blank_close (metadata); /* GeoLocation */
+
+			tracker_sparql_builder_insert_open (preupdate, NULL);
+			tracker_sparql_builder_subject_iri (preupdate, addruri);
+
+			g_free (addruri);
+
+			tracker_sparql_builder_predicate (preupdate, "a");
+			tracker_sparql_builder_object (preupdate, "nco:PostalAddress");
 
 			if (xd->address) {
-				tracker_sparql_builder_predicate (metadata, "mlo:address");
-				tracker_sparql_builder_object_unvalidated (metadata, xd->address);
+				tracker_sparql_builder_predicate (preupdate, "nco:streetAddress");
+				tracker_sparql_builder_object_unvalidated (preupdate, xd->address);
 			}
 
 			if (xd->state) {
-				tracker_sparql_builder_predicate (metadata, "mlo:state");
-				tracker_sparql_builder_object_unvalidated (metadata, xd->state);
+				tracker_sparql_builder_predicate (preupdate, "nco:region");
+				tracker_sparql_builder_object_unvalidated (preupdate, xd->state);
 			}
 
 			if (xd->city) {
-				tracker_sparql_builder_predicate (metadata, "mlo:city");
-				tracker_sparql_builder_object_unvalidated (metadata, xd->city);
+				tracker_sparql_builder_predicate (preupdate, "nco:locality");
+				tracker_sparql_builder_object_unvalidated (preupdate, xd->city);
 			}
 
 			if (xd->country) {
-				tracker_sparql_builder_predicate (metadata, "mlo:country");
-				tracker_sparql_builder_object_unvalidated (metadata, xd->country);
+				tracker_sparql_builder_predicate (preupdate, "nco:country");
+				tracker_sparql_builder_object_unvalidated (preupdate, xd->country);
 			}
 
-			tracker_sparql_builder_object_blank_close (metadata);
+			tracker_sparql_builder_object_blank_close (preupdate);
 		}
 
-		/* PDF keywords aren't used ATM (why not?) */
-		g_free (pd.keywords);
-
-		g_free (pd.title);
-		g_free (pd.subject);
-		g_free (pd.creation_date);
-		g_free (pd.author);
-		g_free (pd.date);
-
 		tracker_xmp_free (xd);
 	} else {
 		/* So if we are here we have NO XMP data and we just
 		 * write what we know from Poppler.
 		 */
-		write_pdf_data (pd, metadata);
+		write_pdf_data (pd, metadata, keywords);
 	}
 
+	for (i = 0; i < keywords->len; i++) {
+		gchar *p;
+
+		p = (gchar *) g_ptr_array_index (keywords, i);
+
+		tracker_sparql_builder_predicate (metadata, "nao:hasTag");
+
+		tracker_sparql_builder_object_blank_open (metadata);
+		tracker_sparql_builder_predicate (metadata, "a");
+		tracker_sparql_builder_object (metadata, "nao:Tag");
+		tracker_sparql_builder_predicate (metadata, "nao:prefLabel");
+		tracker_sparql_builder_object_unvalidated (metadata, p);
+		tracker_sparql_builder_object_blank_close (metadata);
+		g_free (p);
+	}
+	g_ptr_array_free (keywords, TRUE);
+
 	tracker_sparql_builder_predicate (metadata, "nfo:pageCount");
 	tracker_sparql_builder_object_int64 (metadata, poppler_document_get_n_pages (document));
 
-	fts_config = tracker_main_get_fts_config ();
-	n_words = tracker_fts_config_get_max_words_to_index (fts_config);
-	content = extract_content (document, n_words);
+	config = tracker_main_get_config ();
+	n_bytes = tracker_config_get_max_bytes (config);
+	content = extract_content (document, n_bytes);
 
 	if (content) {
 		tracker_sparql_builder_predicate (metadata, "nie:plainTextContent");
@@ -583,6 +597,14 @@ extract_pdf (const gchar          *uri,
 
 	read_outline (document, metadata);
 
+	g_free (xml);
+	g_free (pd.keywords);
+	g_free (pd.title);
+	g_free (pd.subject);
+	g_free (pd.creation_date);
+	g_free (pd.author);
+	g_free (pd.date);
+
 	g_object_unref (document);
 }
 
