@@ -60,6 +60,7 @@ const TestInfo tests[] = {
 	{ "algebra/filter-in-2", "algebra/data-2", FALSE },
 	{ "algebra/filter-in-3", "algebra/data-2", FALSE },
 	{ "algebra/filter-in-4", "algebra/data-2", FALSE },
+	{ "algebra/filter-in-5", "algebra/data-2", FALSE },
 	{ "algebra/var-scope-join-1", "algebra/var-scope-join-1", FALSE },
 	{ "anon/query", "anon/data", FALSE },
 	{ "ask/ask-1", "ask/data", FALSE },
@@ -129,7 +130,7 @@ strstr_i (const char *a, const char *b)
 }
 
 static void
-check_result (TrackerDBResultSet *result_set,
+check_result (TrackerDBCursor *cursor,
               const TestInfo *test_info,
               const gchar *results_filename,
               GError *error)
@@ -155,42 +156,25 @@ check_result (TrackerDBResultSet *result_set,
 
 	test_results = g_string_new ("");
 
-	if (result_set) {
-		gboolean valid = TRUE;
-		guint col_count;
+	if (cursor) {
 		gint col;
 
-		col_count = tracker_db_result_set_get_n_columns (result_set);
+		while (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
+			for (col = 0; col < tracker_db_cursor_get_n_columns (cursor); col++) {
+				const gchar *str;
 
-		while (valid) {
-			for (col = 0; col < col_count; col++) {
-				GValue value = { 0 };
-
-				_tracker_db_result_set_get_value (result_set, col, &value);
-
-				switch (G_VALUE_TYPE (&value)) {
-				case G_TYPE_INT64:
-					g_string_append_printf (test_results, "\"%" G_GINT64_FORMAT "\"", g_value_get_int64 (&value));
-					break;
-				case G_TYPE_DOUBLE:
-					g_string_append_printf (test_results, "\"%f\"", g_value_get_double (&value));
-					break;
-				case G_TYPE_STRING:
-					g_string_append_printf (test_results, "\"%s\"", g_value_get_string (&value));
-					break;
-				default:
-					/* unbound variable */
-					break;
+				if (col > 0) {
+					g_string_append (test_results, "\t");
 				}
 
-				if (col < col_count - 1) {
-					g_string_append (test_results, "\t");
+				str = tracker_db_cursor_get_string (cursor, col, NULL);
+				if (str != NULL) {
+					/* bound variable */
+					g_string_append_printf (test_results, "\"%s\"", str);
 				}
 			}
 
 			g_string_append (test_results, "\n");
-
-			valid = tracker_db_result_set_iter_next (result_set);
 		}
 	} else if (test_info->expect_query_error) {
 		g_string_append (test_results, error->message);
@@ -228,7 +212,7 @@ check_result (TrackerDBResultSet *result_set,
 static void
 test_sparql_query (gconstpointer test_data)
 {
-	TrackerDBResultSet *result_set;
+	TrackerDBCursor *cursor;
 	const TestInfo *test_info;
 	GError *error;
 	gchar *data_filename;
@@ -252,7 +236,9 @@ test_sparql_query (gconstpointer test_data)
 
 	tracker_data_manager_init (TRACKER_DB_MANAGER_FORCE_REINDEX,
 	                           test_schemas,
-	                           NULL, FALSE, 100, 100, NULL, NULL, NULL);
+	                           NULL, FALSE, 100, 100, NULL, NULL, NULL, &error);
+
+	g_assert_no_error (error);
 
 	/* data_path = g_build_path (G_DIR_SEPARATOR_S, TOP_SRCDIR, "tests", "libtracker-data", NULL); */
 
@@ -290,28 +276,28 @@ test_sparql_query (gconstpointer test_data)
 
 	/* perform actual query */
 
-	result_set = tracker_data_query_sparql (query, &error);
+	cursor = tracker_data_query_sparql_cursor (query, &error);
 
-	check_result (result_set, test_info, results_filename, error);
+	check_result (cursor, test_info, results_filename, error);
 
 	g_free (query_filename);
 	g_free (query);
 
 	query_filename = g_strconcat (test_prefix, ".extra.rq", NULL);
 	if (g_file_get_contents (query_filename, &query, NULL, NULL)) {
-		g_object_unref (result_set);
-		result_set = tracker_data_query_sparql (query, &error);
+		g_object_unref (cursor);
+		cursor = tracker_data_query_sparql_cursor (query, &error);
 		g_assert_no_error (error);
 		g_free (results_filename);
 		results_filename = g_strconcat (test_prefix, ".extra.out", NULL);
-		check_result (result_set, test_info, results_filename, error);
+		check_result (cursor, test_info, results_filename, error);
 	}
 
 	g_free (data_prefix);
 	g_free (test_prefix);
 
-	if (result_set) {
-		g_object_unref (result_set);
+	if (cursor) {
+		g_object_unref (cursor);
 	}
 
 	/* cleanup */
@@ -354,6 +340,17 @@ main (int argc, char **argv)
 	for (i = 0; tests[i].test_name; i++) {
 		gchar *testpath;
 
+#ifndef HAVE_LIBICU
+		/* Skip tests which fail collation tests and are known
+		 * to do so. For more details see:
+		 *
+		 * https://bugzilla.gnome.org/show_bug.cgi?id=636074
+		 */
+		if (strcmp (tests[i].test_name, "functions/functions-xpath-2") == 0) {
+			continue;
+		}
+#endif
+
 		testpath = g_strconcat ("/libtracker-data/sparql/", tests[i].test_name, NULL);
 		g_test_add_data_func (testpath, &tests[i], test_sparql_query);
 		g_free (testpath);
